/*
 * Copyright (c) 2005 DocJava, Inc. All Rights Reserved.
 */
package gui.run;

// Fig. 25.33: SliderFieldPanel.java
// A subclass of JPanel containing a JSlider and a JTextField


import gui.ClosableJFrame;
import gui.layouts.DialogLayout;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.AffineTransform;

/**
 * Only works for integers. Shows a textfield and a slider, as well as a
 * label. Shortcut mnemonics are used in the label to focus onto the
 * textfield.
 *
 * @see RunSlider
 */
public abstract class RunAffinePanel
        extends JTabbedPane implements Runnable, MouseListener {

    private AffineTransform at = new AffineTransform();
    private double rotation = 0;
    private double xc = 0;
    private double yc = 0;
    private double shx = 0;
    private double shy = 0;
    private double scx = 100;
    private double scy = 100;
    SpinnerNumberModel xcNm = new SpinnerNumberModel(0, 0, 1024, 1);
    SpinnerNumberModel ycNm = new SpinnerNumberModel(0, 0, 1024, 1);
    private int operationOrderCode = 0;

    public RunAffinePanel() {
        setToolTipText("A Fine Panel!");
        addTab("Rotate",
                null,
                getRotationPanel(),
                "Rotate");
        addTab("Shear",
                null,
                getShearPanel(),
                "Shear");
        addTab("Scale",
                null,
                getScalePanel(),
                "Scale");
        addTab("CenterPoint",
                null,
                getCenterPointPanel(),
                "CenterPoint");
        addTab("Order",
                null,
                getOrderPanel(),
                "Order");
    }

    private JPanel getOrderPanel() {
        String items[] = {
            "scale shear rotate",
            "scale rotate shear",
            "shear scale rotate",
            "shear rotate scale",
            "rotate shear scale",
            "rotate scale shear"
        };
        JPanel jp = new JPanel();
        jp.add(new JLabel("select order of operations"),
                BorderLayout.NORTH);
        jp.add(new RunComboBox(items) {
            public void run() {
                operationOrderCode = getValue();
            }
        },
                BorderLayout.CENTER);
        return
                jp;
    }

    public void setCenterPoint(int xc, int yc) {
        this.xc = xc;
        this.yc = yc;
        xcNm.setValue(new Integer(xc));
        ycNm.setValue(new Integer(yc));
    }

    private JPanel getCenterPointPanel() {
        JPanel jp = new JPanel();

        jp.setLayout(new DialogLayout());
        jp.add(new JLabel("xc"));

        jp.add(new RunSpinnerSlider(xcNm) {
            public void run() {
                xc = getValue();
                RunAffinePanel.this.run();
            }
        });
        jp.add(new JLabel("yc"));

        jp.add(new RunSpinnerSlider(ycNm) {
            public void run() {
                yc = getValue();
                RunAffinePanel.this.run();
            }
        });

        return jp;
    }

    private JPanel getScalePanel() {
        JPanel jp = new JPanel();

        jp.setLayout(new DialogLayout());
        jp.add(new JLabel("scx"));
        jp.add(new RunSpinnerSlider(
                new SpinnerNumberModel(100, 0, 200, 1)) {
            public void run() {
                scx = getValue();
                RunAffinePanel.this.run();
            }
        });
        jp.add(new JLabel("scy"));
        jp.add(new RunSpinnerSlider(
                new SpinnerNumberModel(100, 0, 200, 1)) {
            public void run() {
                scy = getValue();
                RunAffinePanel.this.run();
            }
        });

        return jp;
    }

    private JPanel getShearPanel() {
        JPanel jp = new JPanel();

        jp.setLayout(new DialogLayout());
        jp.add(new JLabel("shx"));
        jp.add(new RunSpinnerSlider(
                new SpinnerNumberModel(1, -200, 200, 0)) {
            public void run() {
                shx = getValue();
                RunAffinePanel.this.run();
            }
        });
        jp.add(new JLabel("shy"));
        jp.add(new RunSpinnerSlider(
                new SpinnerNumberModel(1, -200, 200, 1)) {
            public void run() {
                shy = getValue();
                RunAffinePanel.this.run();
            }
        });
        return jp;
    }

    private JPanel getRotationPanel() {
        JPanel jp = new JPanel();
        jp.setLayout(new DialogLayout());
        jp.add(new JLabel("rotation"));
        jp.add(new RunSpinnerSlider(
                new SpinnerNumberModel(0, -180, 180, 1)) {
            public void run() {
                rotation = getValue() * Math.PI / 180.0;
                RunAffinePanel.this.run();
            }
        });
        return jp;
    }

    public AffineTransform getValue() {
        /*
        String items[] = {
        "scaleShearRotate",0
        "scaleRotateShear",1
        "shearScaleRotate",2
        "shearRotateScale",3
        "rotateShearScale",4
        "rotateScaleShear" 5
        };
        */
        switch (operationOrderCode) {

            case 0:
                return scaleShearRotate();
            case 1:
                return scaleRotateShear();
            case 2:
                return shearScaleRotate();
            case 3:
                return shearRotateScale();
            case 4:
                return rotateShearScale();
            case 5:
                return rotateScaleShear();
        }
        return scaleShearRotate();
    }

    private AffineTransform shearScaleRotate() {
        at.setToTranslation(xc, yc);
        at.shear(shx / 100.0, shy / 100.0);
        at.scale(scx / 100.0, scy / 100.0);
        at.rotate(rotation);
        at.translate(-xc, -yc);
        return at;
    }

    private AffineTransform shearRotateScale() {
        at.setToTranslation(xc, yc);
        at.rotate(rotation);
        at.shear(shx / 100.0, shy / 100.0);
        at.scale(scx / 100.0, scy / 100.0);
        at.translate(-xc, -yc);
        return at;
    }

    private AffineTransform rotateShearScale() {
        at.setToTranslation(xc, yc);
        at.rotate(rotation);
        at.shear(shx / 100.0, shy / 100.0);
        at.scale(scx / 100.0, scy / 100.0);
        at.translate(-xc, -yc);
        return at;
    }

    private AffineTransform rotateScaleShear() {
        at.setToTranslation(xc, yc);
        at.rotate(rotation);
        at.scale(scx / 100.0, scy / 100.0);
        at.shear(shx / 100.0, shy / 100.0);
        at.translate(-xc, -yc);
        return at;
    }

    private AffineTransform scaleRotateShear() {
        at.setToTranslation(xc, yc);
        at.scale(scx / 100.0, scy / 100.0);
        at.rotate(rotation);
        at.shear(shx / 100.0, shy / 100.0);

        at.translate(-xc, -yc);
        return at;
    }

    private AffineTransform scaleShearRotate() {
        at.setToTranslation(xc, yc);
        at.scale(scx / 100.0, scy / 100.0);
        at.shear(shx / 100.0, shy / 100.0);
        at.rotate(rotation);
        at.translate(-xc, -yc);
        return at;
    }


    public static void main(String[] args) {
        ClosableJFrame cf = new ClosableJFrame();
        Container c = cf.getContentPane();
        c.setLayout(new FlowLayout());
        c.add(new RunAffinePanel() {
            public void run() {
                AffineTransform at = getValue();
                System.out.println(at);
            }
        });

        cf.setSize(400, 400);
        cf.show();
    }

    public void mouseClicked(MouseEvent e) {
        this.setCenterPoint(e.getX(), e.getY());
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

}
